unit MMMainFrm;
{------------------------------------------------------------------------------}
{
  Very simple "mail merging" application.
  It loads a template from TEMPLATE.RVF, and data from CUSTOMERS.TXT.
  Template can be modified with another application in this directory:
  TEMPLATEEDITOR.
  This demo is a modified version of the demo in
  Demos\Delphi\Assorted\Fields\MailMerge\

  The main difference - it does not use a predefined set of styles.
  The original demo assumed that fields have StyleNo = 1. It modified this
  text style so that fields in rvOutput look like normal text.
  This demo load the template as it is and does not modify text styles.
  It detects fields by their protection options (checks if rvprModifyProtect
  is in Protection)
}
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVScroll, RichView, StdCtrls, RVStyle, RVTable, CRVData, RVTypes;

type
  TForm1 = class(TForm)
    ListBox1: TListBox;
    rvOutput: TRichView;
    rvsOutput: TRVStyle;
    rvsTemplate: TRVStyle;
    rvTemplate: TRichView;
    Label1: TLabel;
    Label2: TLabel;
    lblFieldsCount: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    Codes: TStringList;
    procedure LoadCustomers;
    function GetFieldValueFromDatabase(
      const FieldName: TRVUnicodeString): TRVUnicodeString;
    procedure FillFields(RVData: TCustomRVData);
    function GetFieldsCount(RVData: TCustomRVData): Integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

{------------------------------------------------------------------------------}
{
  This procedure loads file CUSTOMERS.TXT into:
  - ListBox1.Items (<- names of customers)
  - Codes (<- codes of customers)

  In real applications data can be stored in some database
}
procedure TForm1.LoadCustomers;
var F: TextFile;
    i, Count: Integer;
    s: String;
begin
  AssignFile(F, ExtractFilePath(Application.ExeName)+'Customers.txt');
  Reset(F);
  try
    Readln(F, Count);
    for i := 0 to Count-1 do
    begin
      Readln(F, s);
      Codes.Add(s);
      Readln(F, s);
      ListBox1.Items.Add(Trim(s))
    end;
  finally
    CloseFile(F);
  end;
end;
{------------------------------------------------------------------------------}
{
  Initialization: loading template into rvOutput, loading customers data.
}
procedure TForm1.FormCreate(Sender: TObject);
begin
  Codes := TStringList.Create;
  LoadCustomers;
  rvTemplate.LoadRVF(ExtractFilePath(Application.ExeName)+'template.rvf');
  rvTemplate.Format;
  lblFieldsCount.Caption
     := 'Number of fields: '+IntToStr(GetFieldsCount(rvTemplate.RVData));
  if ListBox1.Items.Count>0 then
  begin
    ListBox1.ItemIndex := 0;
    ListBox1.OnClick(nil);
  end;
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormDestroy(Sender: TObject);
begin
  Codes.Free;
  Codes := nil;
end;
{------------------------------------------------------------------------------}
{
  This function returns a current field value by the field name.
  In this example, we have two fields: "name" and "code"
}
function TForm1.GetFieldValueFromDatabase(
  const FieldName: TRVUnicodeString): TRVUnicodeString;
var Index: Integer;
begin
  Index := ListBox1.ItemIndex;
  if FieldName='name' then
    Result := ListBox1.Items[Index]
  else if FieldName='code' then
    Result := Codes[Index]
  else
    Result := '{error: illegal field code}';
end;
{------------------------------------------------------------------------------}
{
  This function iterates through all items in RVData, and if the text if protected
  and its tag contains non-empty text, it calls GetFieldValueFromDatabase(tag) and
  replace this text with the returned value.
  You can move this function to your application unchanged.
  Initial call: FillFields(RichView.RVData);
}
procedure TForm1.FillFields(RVData: TCustomRVData);
var i,r,c: Integer;
    table: TRVTableItemInfo;
    FieldName: TRVUnicodeString;
begin
  for i := 0 to RVData.ItemCount-1 do
    if RVData.GetItem(i) is TRVTableItemInfo then
    begin
      table := TRVTableItemInfo(RVData.GetItem(i));
      for r := 0 to table.RowCount-1 do
        for c := 0 to table.ColCount-1 do
          if table.Cells[r,c]<>nil then
            FillFields(table.Cells[r,c].GetRVData);
    end
    else if (RVData.GetItemStyle(i)>=0) and (rvprModifyProtect in
       RVData.GetRVStyle.TextStyles[RVData.GetItemStyle(i)].Protection) then
    begin
      FieldName := RVData.GetItemTag(i);
      if FieldName<>'' then
        RVData.SetItemTextW(i, GetFieldValueFromDatabase(FieldName));
    end;
end;
{------------------------------------------------------------------------------}
{
  Very similar function - returns a number of fields in the document
}
function TForm1.GetFieldsCount(RVData: TCustomRVData): Integer;
var
  i,r,c: Integer;
  table: TRVTableItemInfo;
begin
  Result := 0;
  for i := 0 to RVData.ItemCount-1 do
    if RVData.GetItem(i) is TRVTableItemInfo then
    begin
      table := TRVTableItemInfo(RVData.GetItem(i));
      for r := 0 to table.RowCount-1 do
        for c := 0 to table.ColCount-1 do
          if table.Cells[r,c]<>nil then
            inc (Result, GetFieldsCount(table.Cells[r,c].GetRVData));
    end
    else if (RVData.GetItemStyle(i)>=0) and (rvprModifyProtect in
       RVData.GetRVStyle.TextStyles[RVData.GetItemStyle(i)].Protection) then
      inc(Result);
end;
{------------------------------------------------------------------------------}
{
  On highlighting a new list box item: updating document in rvOutput.
}
procedure TForm1.ListBox1Click(Sender: TObject);
begin
  rvOutput.Clear;
  if ListBox1.ItemIndex>=0 then
  begin
    rvOutput.LoadRVF(ExtractFilePath(Application.ExeName)+'template.rvf');
    FillFields(rvOutput.RVData);
    rvOutput.Format;
  end;
end;

end.
